/**
 * Informatique Industrielle et Java
 * chapitre 5
 * Projet: lecture et ecriture sur un port de communication
 *
 * Cette classe gre la lecture asynchrone sur un port srie
 * uniquement, elle ne doit pas tre utilise avec des ports 
 * parallles
 *
 * @author F.Mallet
 * @version 2003.04
 */
import javax.comm.CommPort;
import javax.comm.SerialPortEventListener;
import javax.comm.SerialPortEvent;
import javax.comm.SerialPort;
import javax.comm.CommPortIdentifier;
import java.io.InputStream;
import java.io.InputStreamReader;

public class LecteurSerie extends GestionnaireConflits implements SerialPortEventListener {
	private SerialPort portSerie = null;
	
	/**
	 * @param ident reprsente l'identificateur du port surlequel on opre
	 * @param nomAppli est le nom de l'application qui demande la gestion de conflits
	 */
	public LecteurSerie(CommPortIdentifier ident, String nomAppli) {
		super(ident, nomAppli);
	}
	
	public CommPort ouvrePort() {
		portSerie = (SerialPort)super.ouvrePort();
		if (portSerie != null) {
			portSerie.notifyOnDataAvailable(true);
			portSerie.removeEventListener();
			try {
				portSerie.addEventListener(this);
			} catch(java.util.TooManyListenersException tmle) {
				// Cette exception ne devrait jamais tre leve car on enlve 
				// les couteurs avec removeEventListener et un port srie
				// devrait accepter au moins un couteur.
				System.err.println("Le port a trop d'couteurs");
			}
		}
		return portSerie;
	}
	
	public void fermePort() {
		super.fermePort();
		portSerie = null;
	}
	
	// l'interface SerialPortEventListener dclare la mthode serialEvent
	public void serialEvent(SerialPortEvent se) {
	// On traite seulement les vnements qui indique qu'une nouvelle donne
	// est arrive
		if (se.getEventType() == SerialPortEvent.DATA_AVAILABLE) {
			try {
				InputStreamReader lecteur = new InputStreamReader(getInputStream());
				
				// Tant que le flot d'entre contient des donnes, on les lit
				while (lecteur.ready()) {
					int lu = lecteur.read();
					
					// Si la fin du flot est atteinte on interrompt la rptition
					if (lu == -1) {
						break;
					}
					
					// On crit le texte lu sur la sortie standard					
					System.out.print(lu);
				}
				
				// Ne pas oublie de fermer le flot d'entre.
				lecteur.close();
			} catch (Exception ex) {
				System.err.println("Erreur flot d'entre:"+ex.getMessage());
			}			
		}
	}
		
	public InputStream getInputStream() throws Exception {
		if (portSerie == null) {
			throw new Exception("Le port srie n'est pas ouvert");
		}	
		
		InputStream is = portSerie.getInputStream();
		if (is == null) {
			throw new Exception("Le port ouvert n'est pas accessible en lecture");
		}
		return is;
	}		
}
